#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include "browse.h"

static int strcmp_void(const void *s1, const void*s2)
{
  return strcmp((const char *) s1, (const char *) s2);
}

static void free_fileinfo_void(void *p)
{
  free_fileinfo((struct fileinfo*) p);
}

static void free_noeffect(void *p)
{
  p = p;
}

#define MAXBUF 4096

static int svgdeps(const struct fileinfo* fi)
{
  FILE		 *f = fopen(fi->absfn, "r");
  char		 *s = malloc(MAXBUF * sizeof(char));
  char		 *s_dup;
  struct stat	 sb;
  char		 c;
  int k_ini, k = 0, k_skip;
  int state = 0;
  while ((c = fgetc(f)) != EOF)
    {
      if (state < 6 && c == 'h')
	state = 1;
      else
	switch (state)
	  {
	    case 1:
	      if (c == 'r')
		state = 2;
	      else
		state = 0;
	      break;
	    case 2:
	      if (c == 'e')
		state = 3;
	      else
		state = 0;
	      break;
	    case 3:
	      if (c == 'f')
		state = 4;
	      else
		state = 0;
	      break;
	    case 4:
	      if (c == '=')
		state = 5;
	      else
		state = 0;
	      break;
	    case 5:
	      if (c == '"')
		{
		  state = 6;
		  k = -1;;
		}
	      else
		state = 0;
	      break;
	    case 6:
	      if (c != '"')
		{
		  if (k == -1)
		    k = k_ini = 0;
		  s[k] = c;
		  s[k + 1] = '\0';
		  if (k + 1 < MAXBUF)
		    ++k;
		}
	      else
		{
		  k_skip = 0;
		  if (!strncmp(s, "file://", 7))
		    k_skip += 7;
		  if (s[k_skip] != '/' && strlen(s) + 2 + strlen(fi->dn) < MAXBUF)
		    {
		      k_ini += strlen(fi->dn) + 1;
		      s_dup = strdup(s);
		      s = strcpy(s, fi->dn);
		      s = strcat(s, "/");
		      s = strcat(s, s_dup);
		      free(s_dup);
		    }
		  if (!(lstat(s + k_skip, &sb) != -1 && S_ISREG(sb.st_mode)))
		    {
		      if (strncmp(s + k_ini, "#", 1) &&
			  strncmp(s + k_ini, "data:", 5))
			{
			  fprintf(stderr, "Undefined reference to %s in %s\n",
				  s + k_ini,
				  fi->relfn);
			}
		    }
		  else
		    printf(" %s", s + k_skip);
		  k = -1;
		  state = 0;
		}
	      break;
	    default:
	      ;
	  }
    }
  fclose(f);
  free(s);
  return 1;
}

static int containspace(const char *s)
{
  for (; *s != '\0' && *s != ' '; ++s)
    ;
  return *s == ' ';
}

int main(int argc, char* argv[])
{
  char*		 src;
  char*		 dst;
  char*		 format;
  struct s_list* list;
  struct s_list* p;
  struct s_list* lext;
  struct stat	 sb;

  if (argc != 4)
    {
      fprintf(stderr, "Usage: %s srcdir dstdir format\n", argv[0]);
      exit(2);
    }
  src = argv[1];
  dst = argv[2];
  format = argv[3];

  if (src[strlen(src)-1] == '/')
    src[strlen(src)-1] = '\0';
  if (dst[strlen(dst)-1] == '/')
    dst[strlen(dst)-1] = '\0';

  list = browse(argv[0], src);
  lext = NULL;

  for (p = list; p; p = p->next)
    {
      struct fileinfo * fi = p->data;
      list_add_item_sorted(&lext, fi->ext, strcmp_void);
    }

  if (lext)
    {
      printf(".SUFFIXES:");
      for (p = lext; p; p = p->next)
	printf(" %s", ((char *) p->data));
      printf("\n\n");
    }
  for (p = list; p; p = p->next)
    {
      struct fileinfo* fi = (struct fileinfo *) p->data;
      if (!(lstat(fi->absfn, &sb) != -1 && S_ISREG(sb.st_mode)))
	{
	  fprintf(stderr, "Cannot stat %s or file is non regular. Skipped\n", fi->absfn);
	  continue;
	}
      if (containspace(fi->absfn))
	{
	  fprintf(stderr, "Cannot deal with spaces in image filename %s. Skipped\n", fi->absfn);
	  continue;
	}
      printf("%s/%s.%s: %s", dst, fi->relfn_noext, format, fi->absfn);
      if (!strcasecmp(fi->ext, ".svg"))
	svgdeps(fi);
      printf("\n");
      printf("\t@echo \"Generation of image %s.%s\"\n", fi->relfn_noext, format);
      printf("\t@mkdir -p '%s/%s'\n", dst, fi->reldn);
      printf("\t@$(MT_SCRIPTS_PATH)/mt_convert $(MAKEFILE_RULES) %s %s/%s.%s 2>&1 | $(INDENT) | ($(COLORIZEBLUE); true) \n",
	     fi->absfn, dst, fi->relfn_noext, format);
      printf("\n");
    }
  if (list)
    printf("\n\n");
  /*
  printf("images:");
  for (p = list; p; p = p->next)
    {
      struct fileinfo* fi = (struct fileinfo *) p->data;
      if (!(lstat(fi->absfn, &sb) != -1 && S_ISREG(sb.st_mode)))
	continue;
      if (containspace(fi->absfn))
	continue;
      printf(" %s/%s.%s", dst, fi->relfn_noext, format);
    }
  printf("\n");
  printf("\t@printf \"Image generation success\\\\n\" | ${COLORIZEGREEN}\n");
  printf("\n");
  */

  list_free(&list, free_fileinfo_void);
  list_free(&lext, free_noeffect);

  return EXIT_SUCCESS;
}
